import { DEFAULT_ERROR } from '@/constants/errors';
import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { unwrapResult } from '@reduxjs/toolkit';
import { HttpStatusCode } from 'axios';
import bioEntityContentsReducer from './bioEntity.slice';
import { getBioEntity } from './bioEntity.thunks';
import { BioEntityContentsState } from './bioEntity.types';

const mockedAxiosClient = mockNetworkNewAPIResponse();
const SEARCH_QUERY = 'park7';

jest.mock('../../utils/error-report/errorReporting');

const INITIAL_STATE: BioEntityContentsState = {
  data: [],
  loading: 'idle',
  isContentTabOpened: false,
  error: { name: '', message: '' },
  submapConnections: {
    data: [],
    loading: 'idle',
    error: { name: '', message: '' },
  },
};

describe('bioEntity reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<BioEntityContentsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('bioEntity', bioEntityContentsReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(bioEntityContentsReducer(undefined, action)).toEqual(INITIAL_STATE);
  });

  it('should update store after succesfull getBioEntity query', async () => {
    mockedAxiosClient
      .onGet(
        apiPath.getBioEntityContentsStringWithQuery({
          searchQuery: SEARCH_QUERY,
          isPerfectMatch: false,
        }),
      )
      .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

    const { type } = await store.dispatch(
      getBioEntity({
        searchQuery: SEARCH_QUERY,
        isPerfectMatch: false,
      }),
    );
    const { data } = store.getState().bioEntity;
    const bioEnityWithSearchElement = data.find(
      bioEntity => bioEntity.searchQueryElement === SEARCH_QUERY,
    );

    expect(type).toBe('project/getBioEntityContents/fulfilled');
    expect(bioEnityWithSearchElement).toEqual({
      searchQueryElement: SEARCH_QUERY,
      data: bioEntityResponseFixture.content,
      loading: 'succeeded',
      error: DEFAULT_ERROR,
    });
  });

  it('should update store after failed getBioEntity query', async () => {
    mockedAxiosClient
      .onGet(
        apiPath.getBioEntityContentsStringWithQuery({
          searchQuery: SEARCH_QUERY,
          isPerfectMatch: false,
        }),
      )
      .reply(HttpStatusCode.NotFound, bioEntityResponseFixture);

    const action = await store.dispatch(
      getBioEntity({
        searchQuery: SEARCH_QUERY,
        isPerfectMatch: false,
      }),
    );
    const { data } = store.getState().bioEntity;

    const bioEntityWithSearchElement = data.find(
      bioEntity => bioEntity.searchQueryElement === SEARCH_QUERY,
    );
    expect(action.type).toBe('project/getBioEntityContents/rejected');
    expect(() => unwrapResult(action)).toThrow(
      "Failed to fetch bio entity: The page you're looking for doesn't exist. Please verify the URL and try again.",
    );
    expect(bioEntityWithSearchElement).toEqual({
      searchQueryElement: SEARCH_QUERY,
      data: undefined,
      loading: 'failed',
      error: DEFAULT_ERROR,
    });
  });

  it('should update store on loading getBioEntity query', async () => {
    mockedAxiosClient
      .onGet(
        apiPath.getBioEntityContentsStringWithQuery({
          searchQuery: SEARCH_QUERY,
          isPerfectMatch: false,
        }),
      )
      .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

    const bioEntityContentsPromise = store.dispatch(
      getBioEntity({
        searchQuery: SEARCH_QUERY,
        isPerfectMatch: false,
      }),
    );

    const { data } = store.getState().bioEntity;
    const bioEntityWithSearchElement = data.find(
      bioEntity => bioEntity.searchQueryElement === SEARCH_QUERY,
    );

    expect(bioEntityWithSearchElement).toEqual({
      searchQueryElement: SEARCH_QUERY,
      data: undefined,
      loading: 'pending',
      error: DEFAULT_ERROR,
    });

    bioEntityContentsPromise.then(() => {
      const { data: dataPromiseFulfilled } = store.getState().bioEntity;

      const bioEntityWithSearchElementFulfilled = dataPromiseFulfilled.find(
        bioEntity => bioEntity.searchQueryElement === SEARCH_QUERY,
      );

      expect(bioEntityWithSearchElementFulfilled).toEqual({
        searchQueryElement: SEARCH_QUERY,
        data: bioEntityResponseFixture.content,
        loading: 'succeeded',
        error: DEFAULT_ERROR,
      });
    });
  });
});
